@@ -18,4 +18,9 @@ NULL
 
 
 #' @import methods
-NULL
\ No newline at end of file
+NULL
+
+
+.onAttach <- function(libname, pkgname) {
+      packageStartupMessage("BacArena paper: https://doi.org/10.1371/journal.pcbi.1005544\n Tutorial: https://CRAN.R-project.org/package=BacArena/vignettes/BacArena-Introduction.pdf\n Development and help: https://github.com/euba/bacarena")
+}
